package com.company.testmd.view.topic;

import com.company.testmd.entity.Topic;
import com.company.testmd.view.main.MainView;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.HasValueAndElement;
import com.vaadin.flow.component.formlayout.FormLayout;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.Route;
import io.jmix.core.validation.group.UiCrossFieldChecks;
import io.jmix.flowui.component.UiComponentUtils;
import io.jmix.flowui.component.validation.ValidationErrors;
import io.jmix.flowui.kit.action.ActionPerformedEvent;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.model.CollectionContainer;
import io.jmix.flowui.model.DataContext;
import io.jmix.flowui.model.InstanceContainer;
import io.jmix.flowui.model.InstanceLoader;
import io.jmix.flowui.view.*;

@Route(value = "topics", layout = MainView.class)
@ViewController("Topic.list")
@ViewDescriptor("topic-list-view.xml")
@LookupComponent("topicsDataGrid")
@DialogMode(width = "64em")
public class TopicListView extends StandardListView<Topic> {

    @ViewComponent
    private DataContext dataContext;

    @ViewComponent
    private CollectionContainer<Topic> topicsDc;

    @ViewComponent
    private InstanceContainer<Topic> topicDc;

    @ViewComponent
    private InstanceLoader<Topic> topicDl;

    @ViewComponent
    private VerticalLayout listLayout;

    @ViewComponent
    private FormLayout form;

    @ViewComponent
    private HorizontalLayout detailActions;

    @Subscribe
    public void onBeforeShow(final BeforeShowEvent event) {
        updateControls(false);
    }

    @Subscribe("topicsDataGrid.create")
    public void onTopicsDataGridCreate(final ActionPerformedEvent event) {
        dataContext.clear();
        Topic entity = dataContext.create(Topic.class);
        topicDc.setItem(entity);
        updateControls(true);
    }

    @Subscribe("topicsDataGrid.edit")
    public void onTopicsDataGridEdit(final ActionPerformedEvent event) {
        updateControls(true);
    }

    @Subscribe("saveBtn")
    public void onSaveButtonClick(final ClickEvent<JmixButton> event) {
        Topic item = topicDc.getItem();
        ValidationErrors validationErrors = validateView(item);
        if (!validationErrors.isEmpty()) {
            ViewValidation viewValidation = getViewValidation();
            viewValidation.showValidationErrors(validationErrors);
            viewValidation.focusProblemComponent(validationErrors);
            return;
        }
        dataContext.save();
        topicsDc.replaceItem(item);
        updateControls(false);
    }

    @Subscribe("cancelBtn")
    public void onCancelButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.clear();
        topicDl.load();
        updateControls(false);
    }

    @Subscribe(id = "topicsDc", target = Target.DATA_CONTAINER)
    public void onTopicsDcItemChange(final InstanceContainer.ItemChangeEvent<Topic> event) {
        Topic entity = event.getItem();
        dataContext.clear();
        if (entity != null) {
            topicDl.setEntityId(entity.getId());
            topicDl.load();
        } else {
            topicDl.setEntityId(null);
            topicDc.setItem(null);
        }
        updateControls(false);
    }

    protected ValidationErrors validateView(Topic entity) {
        ViewValidation viewValidation = getViewValidation();
        ValidationErrors validationErrors = viewValidation.validateUiComponents(form);
        if (!validationErrors.isEmpty()) {
            return validationErrors;
        }
        validationErrors.addAll(viewValidation.validateBeanGroup(UiCrossFieldChecks.class, entity));
        return validationErrors;
    }

    private void updateControls(boolean editing) {
        UiComponentUtils.getComponents(form).forEach(component -> {
            if (component instanceof HasValueAndElement<?, ?> field) {
                field.setReadOnly(!editing);
            }
        });

        detailActions.setVisible(editing);
        listLayout.setEnabled(!editing);
    }

    private ViewValidation getViewValidation() {
        return getApplicationContext().getBean(ViewValidation.class);
    }
}